/*
  $Id: large_test.c,v 1.3 2001/07/09 11:41:36 kpchen Exp $
  A program to count the correct ratio of the Chewing input method.
  It is a independent program from the xcin.  It must be compiled with
  "MakeLargeTest"
 */
#define DEBUG

#include<assert.h>
#include<curses.h>
#include<stdio.h>
#include<string.h>
#include<time.h>
#include"chewingio.h"
#include"global.h"
#include"console_chewing.h"
#include"char.h"
#include"dict.h"
#include"zuin.h"
#include"hash.h"

// global data
int nTotalAll, nTotalDiff ;

// extra needed variables & functions
extern const char *ph_str ;
extern char *key_str[MAX_KBTYPE] ;
extern int IsDefPhoEndKey(int, int) ;

// global data
FILE *fpout;
#ifdef DEBUG
FILE *fp_g ;
#endif

int lifetime ;

void CommitString(char *resultStr, ChewingOutput *pgo)
{
  int i;

  for(i=0;i<pgo->nCommitStr; i++) {
    // assume byte 3 and byte 4 were zero filled
    assert(pgo->commitStr[i].s[2] == '\0' &&
	   pgo->commitStr[i].s[3] == '\0') ;

    strcat(resultStr, pgo->commitStr[i].s) ;
#ifdef DEBUG
    fprintf(fp_g, "commit %s\n", pgo->commitStr[i].s) ;
    fflush(fp_g) ;
#endif
  }
}

void Live()
{
  static time_t nowtime, lasttime = 0 ;

  nowtime = time(NULL) ;
  if(lasttime != 0)
    lifetime += (nowtime - lasttime) ;
  lasttime = nowtime ;
}


//----------------------------------------------------


int CallSetConfig(ChewingData *pgdata)
{
  ConfigData config ;
  int i ;

  config.selectAreaLen = 40 ;
  config.maxChiSymbolLen = 16 ;  // suitable for testing
  for(i=0; i<9; i++)
    config.selKey[i] = i + '1' ;
  config.selKey[9] = '0' ;
  SetConfig(pgdata, &config) ;
  return 0 ;
}


int CountBig5Diff(char *s1, char *s2)
{
  int len1, len2, i, nDiff=0 ;

  len1 = strlen(s1) ;
  len2 = strlen(s2) ;
  assert(len1 == len2) ;
  for(i=0; i<len1; i+=2) {
    if(s1[i]!=s2[i] || s1[i+1]!=s2[i+1])
      nDiff++ ;
  }
  return nDiff ;
}

void CheckData(FILE *, ChewingData *) ;

void CheckAllFile(int argc, char *argv[], ChewingData *pgdata)
{
  int i ;
  FILE *infile ;

  for(i=1; i<argc; i++) {
    infile = fopen(argv[i], "r") ;
    if(infile) {
      CheckData(infile, pgdata) ;
      fclose(infile) ;
    }
#ifdef DEBUG
    fprintf(fp_g, "             %d: %s\n\n", i, argv[i]) ;
    fflush(fp_g) ;
#endif
  }
}

void ShowOutput()
{
  printf("Total: %d , Diff: %d\n", nTotalAll, nTotalDiff) ;
#ifdef DEBUG
  fprintf(fp_g, "Total: %d , Diff: %d\n", nTotalAll, nTotalDiff) ;
#endif
}

int main(int argc, char *argv[])
{
  ChewingData *pgdata;
  ChewingConf cf ;

  cf.kb_type = KB_DEFAULT;
  // do some initialization
#ifdef DEBUG
  fp_g = fopen("debug.txt", "w") ;
#endif
  ReadTree(".") ; 
  InitChar(".") ; 
  InitDict(".") ; 
  ReadHash() ;
  // IO_Init() ;
  Live() ; // 2000.8.28 kpchen
  pgdata = ALC(ChewingData,1) ; 
  assert(pgdata) ;
  InitChewing(pgdata, &cf) ;
  CallSetConfig(pgdata) ;

  CheckAllFile(argc, argv, pgdata) ;
  ShowOutput() ;
  return 0 ;
}

void CheckData(FILE *infile, ChewingData *pgdata)
{
  /* begin to read target file from the file.
   * File format example :
   WORD  BOPOMOFO
   WORD  BOPOMOFO
    .       .
    .       .
    .       .
   WORD  BOPOMOFO
  */
  ChewingOutput gOut ;
  int nDiff, nTotal ;
  int key, key_index, lenBOPOMOFO, i ;
  char str[80], *p1, *p2, patternStr[10], *pFindBOPOMOFO ;
  char originStr[1000], resultStr[1000] ;

  originStr[0] = '\0' ;
  resultStr[0] = '\0' ;
  patternStr[2] = '\0' ;
  while(fgets(str, 80, infile) != NULL) {
    p1 = strtok(str, " \n") ;
    p2 = strtok(NULL, " \n") ;
    if(p1 == NULL || p2 == NULL) continue ;
    strcat(originStr, p1) ;
    lenBOPOMOFO = strlen(p2) / 2;
    for(i=0; i<lenBOPOMOFO; i++) {
      strncpy(patternStr, &p2[i*2], 2) ;
      pFindBOPOMOFO = strstr(ph_str, patternStr) ;
      if(pFindBOPOMOFO == NULL) {
	fprintf(stderr, "BOPOMOFO Error\n") ;
	exit(1) ;
      }
      key_index = ( pFindBOPOMOFO - ph_str ) / 2 ;
      key = key_str[KB_DEFAULT][key_index] ;
      
      // Input Key :
#ifdef DEBUG
      fprintf(fp_g, "%c", key) ;
#endif
      OnKeyDefault(pgdata,key, &gOut);
      // Test whether the string was committed
      if(gOut.nCommitStr) {
	CommitString(resultStr, &gOut) ;
      }
    }
#ifdef DEBUG
    fprintf(fp_g, "\n") ;
    fflush(fp_g) ;
#endif
    // to determine whether the last key is a "end-phone".  If not, 
    // then add a ' '  (space).
    if( ! IsDefPhoEndKey(key, KB_DEFAULT) ) {
      OnKeyDefault(pgdata, ' ', &gOut) ;
      // Test whether the string was committed
      if(gOut.nCommitStr) {
	CommitString(resultStr, &gOut) ;
      }
    }
  }
  OnKeyEnter(pgdata, &gOut) ;
  // Test whether the string was committed
  if(gOut.nCommitStr) {
    CommitString(resultStr, &gOut) ;
  }

  nDiff = CountBig5Diff(originStr, resultStr) ;
  nTotal = strlen(originStr) / 2 ;
#ifdef DEBUG
  fprintf(fp_g, "%s\n%s\nTotal: %d\nDiff: %d\n", originStr, resultStr, nTotal, nDiff) ;
#endif

  // add the global data
  nTotalAll += nTotal ;
  nTotalDiff += nDiff ;
}










